@@ -37,6 +37,7 @@
 
 #include <glib/gi18n.h>
 #include <libpeas/peas-extension-set.h>
+#include <gtksourceview/gtksourcestyleschememanager.h>
 
 #include "gedit-app.h"
 #include "gedit-commands.h"
@@ -52,7 +53,7 @@
 #ifdef OS_OSX
 #include "gedit-app-osx.h"
 #else
-#ifdef OS_WIN32
+#ifdef G_OS_WIN32
 #include "gedit-app-win32.h"
 #else
 #include "gedit-app-x11.h"
@@ -162,7 +163,8 @@ gedit_app_constructor (GType                  gtype,
 }
 
 static gboolean
-gedit_app_last_window_destroyed_impl (GeditApp *app)
+gedit_app_last_window_destroyed_impl (GeditApp    *app,
+                                      GeditWindow *window)
 {
 	return TRUE;
 }
@@ -247,34 +249,6 @@ gedit_app_set_window_title_impl (GeditApp    *app,
 	gtk_window_set_title (GTK_WINDOW (window), title);
 }
 
-static void
-gedit_app_class_init (GeditAppClass *klass)
-{
-	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-
-	object_class->finalize = gedit_app_finalize;
-	object_class->dispose = gedit_app_dispose;
-	object_class->get_property = gedit_app_get_property;
-	object_class->constructor = gedit_app_constructor;
-
-	klass->last_window_destroyed = gedit_app_last_window_destroyed_impl;
-	klass->show_help = gedit_app_show_help_impl;
-	klass->help_link_id = gedit_app_help_link_id_impl;
-	klass->set_window_title = gedit_app_set_window_title_impl;
-
-	g_object_class_install_property (object_class,
-					 PROP_LOCKDOWN,
-					 g_param_spec_flags ("lockdown",
-							     "Lockdown",
-							     "The lockdown mask",
-							     GEDIT_TYPE_LOCKDOWN_MASK,
-							     0,
-							     G_PARAM_READABLE |
-							     G_PARAM_STATIC_STRINGS));
-
-	g_type_class_add_private (object_class, sizeof (GeditAppPrivate));
-}
-
 static gboolean
 ensure_user_config_dir (void)
 {
@@ -299,22 +273,6 @@ ensure_user_config_dir (void)
 	return ret;
 }
 
-static void
-load_accels (void)
-{
-	gchar *filename;
-
-	filename = g_build_filename (gedit_dirs_get_user_config_dir (),
-				     "accels",
-				     NULL);
-	if (filename != NULL)
-	{
-		gedit_debug_message (DEBUG_APP, "Loading keybindings from %s\n", filename);
-		gtk_accel_map_load (filename);
-		g_free (filename);
-	}
-}
-
 static void
 save_accels (void)
 {
@@ -349,37 +307,6 @@ get_page_setup_file (void)
 	return setup;
 }
 
-static void
-load_page_setup (GeditApp *app)
-{
-	gchar *filename;
-	GError *error = NULL;
-
-	g_return_if_fail (app->priv->page_setup == NULL);
-
-	filename = get_page_setup_file ();
-
-	app->priv->page_setup = gtk_page_setup_new_from_file (filename,
-							      &error);
-	if (error)
-	{
-		/* Ignore file not found error */
-		if (error->domain != G_FILE_ERROR ||
-		    error->code != G_FILE_ERROR_NOENT)
-		{
-			g_warning ("%s", error->message);
-		}
-
-		g_error_free (error);
-	}
-
-	g_free (filename);
-
-	/* fall back to default settings */
-	if (app->priv->page_setup == NULL)
-		app->priv->page_setup = gtk_page_setup_new ();
-}
-
 static void
 save_page_setup (GeditApp *app)
 {
@@ -421,37 +348,6 @@ get_print_settings_file (void)
 	return settings;
 }
 
-static void
-load_print_settings (GeditApp *app)
-{
-	gchar *filename;
-	GError *error = NULL;
-
-	g_return_if_fail (app->priv->print_settings == NULL);
-
-	filename = get_print_settings_file ();
-
-	app->priv->print_settings = gtk_print_settings_new_from_file (filename,
-								      &error);
-	if (error)
-	{
-		/* Ignore file not found error */
-		if (error->domain != G_FILE_ERROR ||
-		    error->code != G_FILE_ERROR_NOENT)
-		{
-			g_warning ("%s", error->message);
-		}
-
-		g_error_free (error);
-	}
-
-	g_free (filename);
-
-	/* fall back to default settings */
-	if (app->priv->print_settings == NULL)
-		app->priv->print_settings = gtk_print_settings_new ();
-}
-
 static void
 save_print_settings (GeditApp *app)
 {
@@ -476,86 +372,40 @@ save_print_settings (GeditApp *app)
 }
 
 static void
-extension_added (PeasExtensionSet *extensions,
-		 PeasPluginInfo   *info,
-		 PeasExtension    *exten,
-		 GeditApp         *app)
+gedit_app_quit_impl (GeditApp *app)
 {
-	gedit_app_activatable_activate (GEDIT_APP_ACTIVATABLE (exten));
+	gedit_debug_message (DEBUG_APP, "Quitting\n");
+
+	/* Last window is gone... save some settings and exit */
+	ensure_user_config_dir ();
+
+	save_accels ();
+	save_page_setup (app);
+	save_print_settings (app);
+
+	gtk_main_quit ();
 }
 
 static void
-extension_removed (PeasExtensionSet *extensions,
-		   PeasPluginInfo   *info,
-		   PeasExtension    *exten,
-		   GeditApp         *app)
+load_accels (void)
 {
-	gedit_app_activatable_deactivate (GEDIT_APP_ACTIVATABLE (exten));
-}
+	gchar *filename;
 
-static void
-gedit_app_init (GeditApp *app)
-{
-	app->priv = GEDIT_APP_GET_PRIVATE (app);
-
-	load_accels ();
-
-	/* Load settings */
-	app->priv->settings = gedit_settings_new ();
-	app->priv->window_settings = g_settings_new ("org.gnome.gedit.state.window");
-
-	/* initial lockdown state */
-	app->priv->lockdown = gedit_settings_get_lockdown (GEDIT_SETTINGS (app->priv->settings));
-
-	app->priv->extensions = peas_extension_set_new (PEAS_ENGINE (gedit_plugins_engine_get_default ()),
-							GEDIT_TYPE_APP_ACTIVATABLE,
-							"app", app,
-							NULL);
-
-	g_signal_connect (app->priv->extensions,
-			  "extension-added",
-			  G_CALLBACK (extension_added),
-			  app);
-
-	g_signal_connect (app->priv->extensions,
-			  "extension-removed",
-			  G_CALLBACK (extension_removed),
-			  app);
-
-	peas_extension_set_foreach (app->priv->extensions,
-	                            (PeasExtensionSetForeachFunc) extension_added,
-	                            app);
-}
-
-/**
- * gedit_app_get_default:
- *
- * Returns the #GeditApp object. This object is a singleton and
- * represents the running gedit instance.
- *
- * Return value: (transfer none): the #GeditApp pointer
- */
-GeditApp *
-gedit_app_get_default (void)
-{
-	GType type;
-
-	if (app_instance != NULL)
+	filename = g_build_filename (gedit_dirs_get_user_config_dir (),
+				     "accels",
+				     NULL);
+	if (filename != NULL)
 	{
-		return app_instance;
+		gedit_debug_message (DEBUG_APP, "Loading keybindings from %s\n", filename);
+		gtk_accel_map_load (filename);
+		g_free (filename);
 	}
+}
 
-#ifdef OS_OSX
-	type = GEDIT_TYPE_APP_OSX;
-#else
-#ifdef OS_WIN32
-	type = GEDIT_TYPE_APP_WIN32;
-#else
-	type = GEDIT_TYPE_APP_X11;
-#endif
-#endif
-
-	return GEDIT_APP (g_object_new (type, NULL));
+static void
+gedit_app_constructed (GObject *object)
+{
+	load_accels ();
 }
 
 static void
@@ -601,6 +451,12 @@ window_delete_event (GeditWindow *window,
 	return TRUE;
 }
 
+void
+_gedit_app_quit (GeditApp *app)
+{
+	GEDIT_APP_GET_CLASS (app)->quit (app);
+}
+
 static void
 window_destroy (GeditWindow *window, 
 		GeditApp    *app)
@@ -625,22 +481,249 @@ window_destroy (GeditWindow *window,
 */
 	if (app->priv->windows == NULL)
 	{
-		if (!GEDIT_APP_GET_CLASS (app)->last_window_destroyed (app))
+		if (!GEDIT_APP_GET_CLASS (app)->last_window_destroyed (app, window))
 		{
 			return;
 		}
 
-		/* Last window is gone... save some settings and exit */
-		ensure_user_config_dir ();
-
-		save_accels ();
-		save_page_setup (app);
-		save_print_settings (app);
-
-		gtk_main_quit ();
+		_gedit_app_quit (app);
 	}
 }
 
+static GeditWindow *
+gedit_app_create_window_impl (GeditApp *app)
+{
+	GeditWindow *window;
+	gboolean isfirst;
+	
+	/*
+	 * We need to be careful here, there is a race condition:
+	 * when another gedit is launched it checks active_window,
+	 * so we must do our best to ensure that active_window
+	 * is never NULL when at least a window exists.
+	 */
+
+	isfirst = (app->priv->windows == NULL);
+	
+	window = g_object_new (GEDIT_TYPE_WINDOW, NULL);
+
+	if (isfirst)
+	{
+		set_active_window (app, window);
+	}
+
+	app->priv->windows = g_list_prepend (app->priv->windows,
+					     window);
+
+	gedit_debug_message (DEBUG_APP, "Window created");
+
+	g_signal_connect (window, 
+			  "focus_in_event",
+			  G_CALLBACK (window_focus_in_event), 
+			  app);
+	g_signal_connect (window,
+			  "delete_event",
+			  G_CALLBACK (window_delete_event),
+			  app);			  
+	g_signal_connect (window, 
+			  "destroy",
+			  G_CALLBACK (window_destroy),
+			  app);
+
+	return window;
+}
+
+static void
+gedit_app_ready_impl (GeditApp *app)
+{
+}
+
+static void
+gedit_app_class_init (GeditAppClass *klass)
+{
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+	object_class->finalize = gedit_app_finalize;
+	object_class->dispose = gedit_app_dispose;
+	object_class->get_property = gedit_app_get_property;
+	object_class->constructor = gedit_app_constructor;
+	object_class->constructed = gedit_app_constructed;
+
+	klass->last_window_destroyed = gedit_app_last_window_destroyed_impl;
+	klass->show_help = gedit_app_show_help_impl;
+	klass->help_link_id = gedit_app_help_link_id_impl;
+	klass->set_window_title = gedit_app_set_window_title_impl;
+	klass->quit = gedit_app_quit_impl;
+	klass->create_window = gedit_app_create_window_impl;
+	klass->ready = gedit_app_ready_impl;
+
+	g_object_class_install_property (object_class,
+					 PROP_LOCKDOWN,
+					 g_param_spec_flags ("lockdown",
+							     "Lockdown",
+							     "The lockdown mask",
+							     GEDIT_TYPE_LOCKDOWN_MASK,
+							     0,
+							     G_PARAM_READABLE |
+							     G_PARAM_STATIC_STRINGS));
+
+	g_type_class_add_private (object_class, sizeof (GeditAppPrivate));
+}
+
+static void
+load_page_setup (GeditApp *app)
+{
+	gchar *filename;
+	GError *error = NULL;
+
+	g_return_if_fail (app->priv->page_setup == NULL);
+
+	filename = get_page_setup_file ();
+
+	app->priv->page_setup = gtk_page_setup_new_from_file (filename,
+							      &error);
+	if (error)
+	{
+		/* Ignore file not found error */
+		if (error->domain != G_FILE_ERROR ||
+		    error->code != G_FILE_ERROR_NOENT)
+		{
+			g_warning ("%s", error->message);
+		}
+
+		g_error_free (error);
+	}
+
+	g_free (filename);
+
+	/* fall back to default settings */
+	if (app->priv->page_setup == NULL)
+		app->priv->page_setup = gtk_page_setup_new ();
+}
+
+static void
+load_print_settings (GeditApp *app)
+{
+	gchar *filename;
+	GError *error = NULL;
+
+	g_return_if_fail (app->priv->print_settings == NULL);
+
+	filename = get_print_settings_file ();
+
+	app->priv->print_settings = gtk_print_settings_new_from_file (filename,
+								      &error);
+	if (error)
+	{
+		/* Ignore file not found error */
+		if (error->domain != G_FILE_ERROR ||
+		    error->code != G_FILE_ERROR_NOENT)
+		{
+			g_warning ("%s", error->message);
+		}
+
+		g_error_free (error);
+	}
+
+	g_free (filename);
+
+	/* fall back to default settings */
+	if (app->priv->print_settings == NULL)
+		app->priv->print_settings = gtk_print_settings_new ();
+}
+
+static void
+extension_added (PeasExtensionSet *extensions,
+		 PeasPluginInfo   *info,
+		 PeasExtension    *exten,
+		 GeditApp         *app)
+{
+	gedit_app_activatable_activate (GEDIT_APP_ACTIVATABLE (exten));
+}
+
+static void
+extension_removed (PeasExtensionSet *extensions,
+		   PeasPluginInfo   *info,
+		   PeasExtension    *exten,
+		   GeditApp         *app)
+{
+	gedit_app_activatable_deactivate (GEDIT_APP_ACTIVATABLE (exten));
+}
+
+static void
+gedit_app_init (GeditApp *app)
+{
+	GtkSourceStyleSchemeManager *manager;
+
+	app->priv = GEDIT_APP_GET_PRIVATE (app);
+
+	/* Load settings */
+	app->priv->settings = gedit_settings_new ();
+	app->priv->window_settings = g_settings_new ("org.gnome.gedit.state.window");
+
+	/* initial lockdown state */
+	app->priv->lockdown = gedit_settings_get_lockdown (GEDIT_SETTINGS (app->priv->settings));
+
+	/*
+	 * We use the default gtksourceview style scheme manager so that plugins
+	 * can obtain it easily without a gedit specific api, but we need to
+	 * add our search path at startup before the manager is actually used.
+	 */
+	manager = gtk_source_style_scheme_manager_get_default ();
+	gtk_source_style_scheme_manager_append_search_path (manager,
+							    gedit_dirs_get_user_styles_dir ());
+
+	app->priv->extensions = peas_extension_set_new (PEAS_ENGINE (gedit_plugins_engine_get_default ()),
+							GEDIT_TYPE_APP_ACTIVATABLE,
+							"app", app,
+							NULL);
+
+	g_signal_connect (app->priv->extensions,
+			  "extension-added",
+			  G_CALLBACK (extension_added),
+			  app);
+
+	g_signal_connect (app->priv->extensions,
+			  "extension-removed",
+			  G_CALLBACK (extension_removed),
+			  app);
+
+	peas_extension_set_foreach (app->priv->extensions,
+	                            (PeasExtensionSetForeachFunc) extension_added,
+	                            app);
+}
+
+/**
+ * gedit_app_get_default:
+ *
+ * Returns the #GeditApp object. This object is a singleton and
+ * represents the running gedit instance.
+ *
+ * Return value: (transfer none): the #GeditApp pointer
+ */
+GeditApp *
+gedit_app_get_default (void)
+{
+	GType type;
+
+	if (app_instance != NULL)
+	{
+		return app_instance;
+	}
+
+#ifdef OS_OSX
+	type = GEDIT_TYPE_APP_OSX;
+#else
+#ifdef G_OS_WIN32
+	type = GEDIT_TYPE_APP_WIN32;
+#else
+	type = GEDIT_TYPE_APP_X11;
+#endif
+#endif
+
+	return GEDIT_APP (g_object_new (type, NULL));
+}
+
 /* Generates a unique string for a window role */
 static gchar *
 gen_role (void)
@@ -663,27 +746,8 @@ gedit_app_create_window_real (GeditApp    *app,
 			      const gchar *role)
 {
 	GeditWindow *window;
-
-	/*
-	 * We need to be careful here, there is a race condition:
-	 * when another gedit is launched it checks active_window,
-	 * so we must do our best to ensure that active_window
-	 * is never NULL when at least a window exists.
-	 */
-	if (app->priv->windows == NULL)
-	{
-		window = g_object_new (GEDIT_TYPE_WINDOW, NULL);
-		set_active_window (app, window);
-	}
-	else
-	{
-		window = g_object_new (GEDIT_TYPE_WINDOW, NULL);
-	}
-
-	app->priv->windows = g_list_prepend (app->priv->windows,
-					     window);
-
-	gedit_debug_message (DEBUG_APP, "Window created");
+	
+	window = GEDIT_APP_GET_CLASS (app)->create_window (app);
 
 	if (role != NULL)
 	{
@@ -709,32 +773,28 @@ gedit_app_create_window_real (GeditApp    *app,
 		g_settings_get (app->priv->window_settings,
 				GEDIT_SETTINGS_WINDOW_SIZE,
 				"(ii)", &w, &h);
+
 		gtk_window_set_default_size (GTK_WINDOW (window), w, h);
 
 		if ((state & GDK_WINDOW_STATE_MAXIMIZED) != 0)
+		{
 			gtk_window_maximize (GTK_WINDOW (window));
+		}
 		else
+		{
 			gtk_window_unmaximize (GTK_WINDOW (window));
+		}
 
 		if ((state & GDK_WINDOW_STATE_STICKY ) != 0)
+		{
 			gtk_window_stick (GTK_WINDOW (window));
+		}
 		else
+		{
 			gtk_window_unstick (GTK_WINDOW (window));
+		}
 	}
 
-	g_signal_connect (window, 
-			  "focus_in_event",
-			  G_CALLBACK (window_focus_in_event), 
-			  app);
-	g_signal_connect (window,
-			  "delete_event",
-			  G_CALLBACK (window_delete_event),
-			  app);			  
-	g_signal_connect (window, 
-			  "destroy",
-			  G_CALLBACK (window_destroy),
-			  app);
-
 	return window;
 }
 
@@ -758,7 +818,9 @@ gedit_app_create_window (GeditApp  *app,
 	window = gedit_app_create_window_real (app, TRUE, NULL);
 
 	if (screen != NULL)
+	{
 		gtk_window_set_screen (GTK_WINDOW (window), screen);
+	}
 
 	return window;
 }
@@ -1017,9 +1079,28 @@ gedit_app_set_window_title (GeditApp    *app,
                             GeditWindow *window,
                             const gchar *title)
 {
+	g_return_if_fail (GEDIT_IS_APP (app));
+	g_return_if_fail (GEDIT_IS_WINDOW (window));
+
 	GEDIT_APP_GET_CLASS (app)->set_window_title (app, window, title);
 }
 
+gboolean
+gedit_app_process_window_event (GeditApp    *app,
+                                GeditWindow *window,
+                                GdkEvent    *event)
+{
+	g_return_val_if_fail (GEDIT_IS_APP (app), FALSE);
+	g_return_val_if_fail (GEDIT_IS_WINDOW (window), FALSE);
+
+	if (GEDIT_APP_GET_CLASS (app)->process_window_event)
+	{
+		return GEDIT_APP_GET_CLASS (app)->process_window_event (app, window, event);
+	}
+
+    return FALSE;
+}
+
 static void
 app_lockdown_changed (GeditApp *app)
 {
@@ -1116,4 +1197,12 @@ _gedit_app_get_settings (GeditApp *app)
 	return app->priv->settings;
 }
 
+void
+_gedit_app_ready (GeditApp *app)
+{
+	g_return_if_fail (GEDIT_IS_APP (app));
+
+	GEDIT_APP_GET_CLASS (app)->ready (app);
+}
+
 /* ex:set ts=8 noet: */
