@@ -31,6 +31,10 @@
 #include "gedit-window.h"
 #include "gedit-utils.h"
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #ifdef GDK_WINDOWING_X11
 #include <gdk/gdkx.h>
 #endif
@@ -101,14 +105,17 @@ struct _GeditDBusPrivate
 
 	guint32 next_wait_id;
 
+#ifdef G_OS_UNIX
 	GeditFifo *stdin_fifo;
 	GInputStream *stdin_in_stream;
 	GOutputStream *stdin_out_stream;
+#endif
 	GCancellable *stdin_cancellable;
 };
 
 G_DEFINE_TYPE (GeditDBus, gedit_dbus, G_TYPE_OBJECT)
 
+#ifdef G_OS_UNIX
 static void
 async_window_destroyed (AsyncData *async,
                         GObject   *where_the_object_was)
@@ -146,10 +153,12 @@ async_data_new (GeditDBus *dbus)
 
 	return async;
 }
+#endif
 
 static void
 gedit_dbus_dispose (GObject *object)
 {
+#ifdef G_OS_UNIX
 	GeditDBus *dbus = GEDIT_DBUS (object);
 
 	if (dbus->priv->stdin_cancellable)
@@ -163,6 +172,7 @@ gedit_dbus_dispose (GObject *object)
 	g_clear_object (&dbus->priv->stdin_fifo);
 	g_clear_object (&dbus->priv->stdin_out_stream);
 	g_clear_object (&dbus->priv->stdin_in_stream);
+#endif
 
 	G_OBJECT_CLASS (gedit_dbus_parent_class)->dispose (object);
 }
@@ -286,7 +296,12 @@ get_display_arguments (GeditDBus         *dbus,
 	screen = gdk_screen_get_default ();
 	display = gdk_screen_get_display (screen);
 
+#ifdef OS_OSX
+	/* On OS X gdk_display_get_name seems to block... */
+	dparams->display_name = NULL;
+#else
 	dparams->display_name = g_strdup (gdk_display_get_name (display));
+#endif
 	dparams->screen_number = gdk_screen_get_number (screen);
 
 	dparams->workspace = gedit_utils_get_current_workspace (screen);
@@ -323,6 +338,7 @@ compose_open_parameters (GeditDBus *dbus)
 	/* Compose additional options */
 	g_variant_builder_init (&options, G_VARIANT_TYPE ("a{sv}"));
 
+#ifdef G_OS_UNIX
 	/* see if we need to add the pipe_path */
 	if (dbus->priv->stdin_fifo)
 	{
@@ -340,6 +356,7 @@ compose_open_parameters (GeditDBus *dbus)
 		g_object_unref (file);
 		g_free (path);
 	}
+#endif
 
 	/* add the encoding, line position, column position */
 	encoding = gedit_command_line_get_encoding (command_line);
@@ -388,10 +405,13 @@ compose_open_parameters (GeditDBus *dbus)
 	/* display parameters like display name, screen, workspace, viewport */
 	get_display_arguments (dbus, &dparams);
 
-	g_variant_builder_add (&options,
-	                       "{sv}",
-	                       "display_name",
-	                       g_variant_new_string (dparams.display_name));
+	if (dparams.display_name)
+	{
+		g_variant_builder_add (&options,
+		                       "{sv}",
+		                       "display_name",
+		                       g_variant_new_string (dparams.display_name));
+	}
 
 	g_free (dparams.display_name);
 
@@ -1656,6 +1676,8 @@ gedit_dbus_run (GeditDBus *dbus)
 		}
 	}
 
+	dbus->priv->main_loop = g_main_loop_new (NULL, FALSE);
+
 	g_bus_own_name (G_BUS_TYPE_SESSION,
 	                "org.gnome.gedit",
 	                G_BUS_NAME_OWNER_FLAGS_NONE,
@@ -1667,7 +1689,6 @@ gedit_dbus_run (GeditDBus *dbus)
 
 	gedit_debug_message (DEBUG_DBUS, "Own name org.gnome.gedit\n");
 
-	dbus->priv->main_loop = g_main_loop_new (NULL, FALSE);
 	g_main_loop_run (dbus->priv->main_loop);
 	g_main_loop_unref (dbus->priv->main_loop);
 
